@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2018, Kovid Goyal <kovid at kovidgoyal.net>
 
@@ -7,6 +7,7 @@ import re
 from functools import lru_cache
 from hashlib import md5
 from mimetypes import guess_type
+from contextlib import suppress
 
 path_name_map = {}
 
@@ -116,12 +117,9 @@ def collect_files(collection, left, right):
 sanitize_pat = re.compile('[\x00-\x09\x0b-\x1f\x7f\x80-\x9f]')
 
 
-def sanitize_sub(m):
-    return '<{:x}>'.format(ord(m.group()[0]))
-
-
 def sanitize(text):
-    return sanitize_pat.sub(sanitize_sub, text)
+    ntext = text.replace('\r\n', '⏎\n')
+    return sanitize_pat.sub('░', ntext)
 
 
 @lru_cache(maxsize=1024)
@@ -143,10 +141,8 @@ def is_image(path):
 def data_for_path(path):
     ans = raw_data_for_path(path)
     if not is_image(path) and not os.path.samefile(path, os.devnull):
-        try:
+        with suppress(UnicodeDecodeError):
             ans = ans.decode('utf-8')
-        except UnicodeDecodeError:
-            pass
     return ans
 
 
