@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2018, Kovid Goyal <kovid at kovidgoyal.net>
 
@@ -8,6 +8,7 @@ import shlex
 import sys
 import traceback
 from functools import lru_cache
+from contextlib import suppress
 
 from .cli import (
     emph, green, italic, parse_option_spec, print_help_for_seq, title
@@ -22,7 +23,7 @@ match_commands = tuple(sorted(all_commands + ('exit', 'help', 'quit')))
 def init_readline(readline):
     try:
         readline.read_init_file()
-    except EnvironmentError:
+    except OSError:
         if not is_macos:
             raise
     if 'libedit' in readline.__doc__:
@@ -44,6 +45,8 @@ def options_for_cmd(cmd):
         func = cmap[cmd]
     except KeyError:
         return (), alias_map
+    if not func.options_spec:
+        return (), alias_map
     seq, disabled = parse_option_spec(func.options_spec)
     ans = []
     for opt in seq:
@@ -66,10 +69,8 @@ class Completer:
     def __init__(self):
         self.matches = []
         ddir = cache_dir()
-        try:
+        with suppress(FileExistsError):
             os.makedirs(ddir)
-        except FileExistsError:
-            pass
         self.history_path = os.path.join(ddir, 'shell.history')
 
     def complete(self, text, state):
@@ -84,7 +85,7 @@ class Completer:
             return self.matches[state]
 
     def __enter__(self):
-        if os.path.exists(self.history_path):
+        with suppress(Exception):
             readline.read_history_file(self.history_path)
         readline.set_completer(self.complete)
         delims = readline.get_completer_delims()
@@ -132,6 +133,7 @@ def run_cmd(global_opts, cmd, func, opts, items):
     send = {
         'cmd': cmd,
         'version': version,
+        'no_response': False,
     }
     if payload is not None:
         send['payload'] = payload
@@ -153,7 +155,10 @@ def real_main(global_opts):
 
     while True:
         try:
-            cmdline = input('🐱 ')
+            try:
+                cmdline = input('🐱 ')
+            except UnicodeEncodeError:
+                cmdline = input('kitty> ')
         except EOFError:
             break
         except KeyboardInterrupt:
